import numpy as np
import matplotlib.pyplot as plt
import pathlib

from PIL import Image
from numba import jit
from stardist import random_label_cmap

np.random.seed(42)
lbl_cmap = random_label_cmap()


def load_jpg(path):
    x = np.array(Image.open(path))
    print(path.name, ":", x.shape, x.dtype, x.min(), x.max())
    
    return x


def load_img(path):
    return load_jpg(path)


def load_mask(path, relabel=True):
    y = load_jpg(path)
    
    # labels are not increasing from 0 to <num_instance>-1 but in some other
    # order so we redo that
        
    if relabel:
        # they are already ordered increasingly
        unique_labels = np.unique(y)
        for i, l in enumerate(unique_labels):
            y[y == l] = i
    
    return y


def plot_img_and_mask(img, lbl, img_title="image", lbl_title="label", **kwargs):
    fig, (ai,al,ab) = plt.subplots(1,3, figsize=(21,8), gridspec_kw=dict(width_ratios=(1.25, 1, 1)), **kwargs)
    
    im = ai.imshow(img, cmap='gray')
    ai.set_title(img_title)    
    fig.colorbar(im, ax=ai, orientation="horizontal")
    
    al.imshow(lbl, cmap=lbl_cmap)
    al.set_title(lbl_title)
    
    ab.imshow(img, cmap="gray")
    ab.imshow(lbl, alpha=0.5, cmap=lbl_cmap)
    ab.set_title("Overlay")
    
    plt.tight_layout()

@jit(nopython=True)
def get_boundary(img,value,neighbors):
    points=list()
    for i in range(len(img)):
        for j in range(len(img[0])):
            if (img[i,j]==value)and(np.count_nonzero(neighbors[i,j])<9):
                points.append([i,j])
    return np.array(points)